@@ -46,6 +46,13 @@
 #include "ulib_std.h"
 #include "ulib_opsc.h"
 
+#if defined(__PLATFORM_WIN32__)
+#include "dirent_win32.h"
+#endif
+//#if defined(__WINDOWS_PTHREAD__)
+#include <sys/stat.h>
+//#endif
+
 using namespace std;
 
 
@@ -53,6 +60,10 @@ using namespace std;
 
 // function prototypes
 t_CKBOOL load_internal_modules( Chuck_Compiler * compiler );
+t_CKBOOL load_external_modules( Chuck_Compiler * compiler, 
+                                const char * extension, 
+                                std::list<std::string> & chugin_search_paths,
+                                std::list<std::string> & named_dls);
 t_CKBOOL load_module( Chuck_Env * env, f_ck_query query, const char * name, const char * nspc );
 
 
@@ -89,7 +100,9 @@ Chuck_Compiler::~Chuck_Compiler()
 // name: initialize()
 // desc: initialize the compiler
 //-----------------------------------------------------------------------------
-t_CKBOOL Chuck_Compiler::initialize( Chuck_VM * vm )
+t_CKBOOL Chuck_Compiler::initialize( Chuck_VM * vm, 
+                                     std::list<std::string> & chugin_search_paths,
+                                     std::list<std::string> & named_dls )
 {
     // log
     EM_log( CK_LOG_SYSTEM, "initializing compiler..." );
@@ -112,6 +125,10 @@ t_CKBOOL Chuck_Compiler::initialize( Chuck_VM * vm )
     if( !load_internal_modules( this ) )
         goto error;
     
+    // load external libs
+    if( !load_external_modules( this, ".chug", chugin_search_paths, named_dls ) )
+        goto error;
+    
     // pop indent
     EM_poplog();
 
@@ -149,6 +166,14 @@ void Chuck_Compiler::shutdown()
     code = NULL;
     m_auto_depend = FALSE;
     m_recent.clear();
+    
+    for(std::list<Chuck_DLL *>::iterator i = m_dlls.begin();
+        i != m_dlls.end(); i++)
+    {
+        delete (*i);
+    }
+    
+    m_dlls.clear();
 
     // pop indent
     EM_poplog();
@@ -176,7 +201,7 @@ void Chuck_Compiler::set_auto_depend( t_CKBOOL v )
 // name: go()
 // desc: parse, type-check, and emit a program
 //-----------------------------------------------------------------------------
-t_CKBOOL Chuck_Compiler::go( const string & filename, FILE * fd, const char * str_src )
+t_CKBOOL Chuck_Compiler::go( const string & filename, FILE * fd, const char * str_src, const string & full_path )
 {
     t_CKBOOL ret = TRUE;
     Chuck_Context * context = NULL;
@@ -184,8 +209,8 @@ t_CKBOOL Chuck_Compiler::go( const string & filename, FILE * fd, const char * st
     // check to see if resolve dependencies automatically
     if( !m_auto_depend )
     {
-        // normal
-        ret = this->do_normal( filename, fd, str_src );
+        // normal (note: full_path added 1.3.0.0)
+        ret = this->do_normal( filename, fd, str_src, full_path );
         return ret;
     }
     else // auto
@@ -367,7 +392,7 @@ t_CKBOOL Chuck_Compiler::do_all_except_classes( Chuck_Context * context )
 // name: do_normal()
 // desc: compile normally without auto-depend
 //-----------------------------------------------------------------------------
-t_CKBOOL Chuck_Compiler::do_normal( const string & filename, FILE * fd, const char * str_src )
+t_CKBOOL Chuck_Compiler::do_normal( const string & filename, FILE * fd, const char * str_src, const string & full_path )
 {
     t_CKBOOL ret = TRUE;
     Chuck_Context * context = NULL;
@@ -379,6 +404,9 @@ t_CKBOOL Chuck_Compiler::do_normal( const string & filename, FILE * fd, const ch
     // make the context
     context = type_engine_make_context( g_program, filename );
     if( !context ) return FALSE;
+    
+    // remember full path (added 1.3.0.0)
+    context->full_path = full_path;
 
     // reset the env
     env->reset();
@@ -488,14 +516,15 @@ t_CKBOOL load_module( Chuck_Env * env, f_ck_query query,
     
     // load osc
     dll = new Chuck_DLL( name );
-    if( !dll->load( query ) || (query_failed = !dll->query()) || 
+    // (fixed: 1.3.0.0) query_failed now catches either failure of load or query
+    if( (query_failed = !(dll->load( query ) && dll->query())) ||
         !type_engine_add_dll( env, dll, nspc ) )
     {
         fprintf( stderr, 
                  "[chuck]: internal error loading module '%s.%s'...\n", 
                  nspc, name );
         if( query_failed )
-            fprintf( stderr, "       %s\n", dll->last_error() );
+            fprintf( stderr, "       %s", dll->last_error() );
 
         return FALSE;
     }
@@ -516,7 +545,7 @@ t_CKBOOL load_internal_modules( Chuck_Compiler * compiler )
     EM_log( CK_LOG_SEVERE, "loading built-in modules..." );
     // push indent level
     EM_pushlog();
-
+    
     // get env
     Chuck_Env * env = compiler->env;
     // make context
@@ -525,7 +554,7 @@ t_CKBOOL load_internal_modules( Chuck_Compiler * compiler )
     env->reset();
     // load it
     type_engine_load_context( env, context );
-
+    
     // load
     EM_log( CK_LOG_SEVERE, "module osc..." );
     load_module( env, osc_query, "osc", "global" );
@@ -539,7 +568,7 @@ t_CKBOOL load_internal_modules( Chuck_Compiler * compiler )
     load_module( env, xform_query, "xform", "global" );
     EM_log( CK_LOG_SEVERE, "module extract..." );
     load_module( env, extract_query, "extract", "global" );
-
+    
     // load
     EM_log( CK_LOG_SEVERE, "class 'machine'..." );
     if( !load_module( env, machine_query, "Machine", "global" ) ) goto error;
@@ -551,32 +580,254 @@ t_CKBOOL load_internal_modules( Chuck_Compiler * compiler )
     EM_log( CK_LOG_SEVERE, "class 'opsc'..." );
     if( !load_module( env, opensoundcontrol_query, "opsc", "global" ) ) goto error;
     // if( !load_module( env, net_query, "net", "global" ) ) goto error;
-
+    
+#ifndef __DISABLE_MIDI__
     if( !init_class_Midi( env ) ) goto error;
     if( !init_class_MidiRW( env ) ) goto error;
+#endif // __DISABLE_MIDI__
     if( !init_class_HID( env ) ) goto error;
-
+        
     // clear context
     type_engine_unload_context( env );
-
+    
     // commit what is in the type checker at this point
     env->global()->commit();
-
+    
     // pop indent level
     EM_poplog();
     
     return TRUE;
-
+    
 error:
-
+    
     // probably dangerous: rollback
     env->global()->rollback();
-
+    
     // clear context
     type_engine_unload_context( env );
-
+    
     // pop indent level
     EM_poplog();
-
+    
     return FALSE;
 }
+
+
+
+//-----------------------------------------------------------------------------
+// name: load_external_module_at_path()
+// desc: ...
+//-----------------------------------------------------------------------------
+t_CKBOOL load_external_module_at_path( Chuck_Compiler * compiler, 
+                                       const char * name,
+                                       const char * dl_path )
+{
+    Chuck_Env * env = compiler->env;
+    
+    EM_log(CK_LOG_SEVERE, "loading chugin '%s'", name);
+    
+    Chuck_DLL * dll = new Chuck_DLL(name);
+    t_CKBOOL query_failed = FALSE;
+    
+    if((query_failed = !(dll->load(dl_path) && dll->query())) ||
+       !type_engine_add_dll2(env, dll, "global"))
+    {
+        EM_log(CK_LOG_SEVERE, "error loading chugin '%s', skipping", name);
+        if(query_failed)
+            EM_log(CK_LOG_SEVERE, "error from chuck_dl: '%s'", dll->last_error());
+        delete dll;
+        
+        return FALSE;
+    }
+    else
+    {
+        compiler->m_dlls.push_back(dll);
+        return TRUE;
+    }        
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: extension_matches()
+// desc: ...
+//-----------------------------------------------------------------------------
+static t_CKBOOL extension_matches( const char * filename, const char * extension )
+{
+    t_CKUINT extension_length = strlen(extension);
+    t_CKUINT filename_length = strlen(filename);
+    
+    return strncmp( extension, filename+(filename_length-extension_length), 
+                    extension_length) == 0;
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: load_external_modules_in_directory()
+// desc: ...
+//-----------------------------------------------------------------------------
+t_CKBOOL load_external_modules_in_directory( Chuck_Compiler * compiler,
+                                             const char * directory,
+                                             const char * extension )
+{
+    static const t_CKBOOL RECURSIVE_SEARCH = false;
+    
+    DIR * dir = opendir(directory);
+    
+    if( dir )
+    {
+        // log
+        EM_log( CK_LOG_INFO, "examining directory '%s' for chugins", directory );
+        
+        struct dirent *de = NULL;
+        
+        while( (de = readdir(dir)) )
+        {
+            t_CKBOOL is_regular = false;
+            t_CKBOOL is_directory = false;
+            
+#if defined(__PLATFORM_WIN32__)
+            is_directory = de->data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY;
+            is_regular = ((de->data.dwFileAttributes & FILE_ATTRIBUTE_NORMAL) || 
+                          (de->data.dwFileAttributes & FILE_ATTRIBUTE_READONLY) ||
+                          (de->data.dwFileAttributes & FILE_ATTRIBUTE_ARCHIVE));
+#elif defined(__WINDOWS_PTHREAD__) // Cygwin -- doesn't have dirent d_type
+            std::string absolute_path = std::string(directory) + "/" + de->d_name;
+            struct stat st;
+            if( stat(absolute_path.c_str(), &st) == 0 )
+            {
+                is_directory = st.st_mode & S_IFDIR;
+                is_regular = st.st_mode & S_IFREG;
+            }
+            else
+            {
+                // uhh ... 
+                EM_log( CK_LOG_INFO, 
+                        "unable to stat file '%s', ignoring for chugins", 
+                        absolute_path.c_str() );
+                continue;
+            }
+#else
+            is_directory = de->d_type == DT_DIR;
+            is_regular = de->d_type == DT_REG;
+#endif
+            
+            if( is_regular ) // TODO: follow links?
+            {
+                if( extension_matches(de->d_name, extension) )
+                {
+                    std::string absolute_path = std::string(directory) + "/" + de->d_name;
+                    
+                    load_external_module_at_path(compiler, de->d_name, 
+                                                 absolute_path.c_str());
+                }
+                else if( extension_matches(de->d_name, ".ck") )
+                {
+                    std::string absolute_path = std::string(directory) + "/" + de->d_name;
+                    compiler->m_cklibs_to_preload.push_back(absolute_path);
+                }
+            }
+            else if( RECURSIVE_SEARCH && is_directory )
+            {
+                // recurse
+                // TODO: max depth?
+                if( strncmp(de->d_name, ".", sizeof(".") != 0) &&
+                    strncmp(de->d_name, "..", sizeof("..")) != 0 )
+                {
+                    std::string absolute_path = std::string(directory) + "/" + de->d_name;
+                    load_external_modules_in_directory(compiler, 
+                                                       absolute_path.c_str(),
+                                                       extension);
+                }
+            }
+        }
+        
+        // close
+        closedir( dir );
+    }
+    else
+    {
+        // log (1.3.1.2: changed to 2 lines to stay within 80 chars)
+        EM_log( CK_LOG_INFO, "unable to open directory '%s'...", directory );
+        EM_log( CK_LOG_INFO, "(ignoring for chugins...)" );
+    }
+    
+    return TRUE;
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: load_external_modules()
+// desc: ...
+//-----------------------------------------------------------------------------
+t_CKBOOL load_external_modules( Chuck_Compiler * compiler, 
+                                const char * extension, 
+                                std::list<std::string> & chugin_search_paths,
+                                std::list<std::string> & named_dls )
+{
+    // log
+    EM_log( CK_LOG_SEVERE, "loading chugins" );
+    // push indent level
+    EM_pushlog();
+    
+    // get env
+    Chuck_Env * env = compiler->env;
+    // make context
+    Chuck_Context * context = type_engine_make_context( NULL, "@[external]" );
+    // reset env - not needed since we just created the env
+    env->reset();
+    // load it
+    type_engine_load_context( env, context );
+    
+    /* first load dynamic libraries explicitly named on the command line */
+    
+    for(std::list<std::string>::iterator i_dl = named_dls.begin();
+        i_dl != named_dls.end(); i_dl++)
+    {
+        std::string & dl_path = *i_dl;
+        if(!extension_matches(dl_path.c_str(), extension))
+            dl_path += extension;
+        
+        load_external_module_at_path(compiler, dl_path.c_str(),
+                                     dl_path.c_str());
+    }
+    
+    /* now recurse through search paths and load any DLs or .ck files found */
+    
+    for(std::list<std::string>::iterator i_sp = chugin_search_paths.begin();
+        i_sp != chugin_search_paths.end(); i_sp++)
+    {
+        load_external_modules_in_directory(compiler, (*i_sp).c_str(), extension);
+    }    
+        
+    // clear context
+    type_engine_unload_context( env );
+    
+    // commit what is in the type checker at this point
+    env->global()->commit();
+    
+    // pop indent level
+    EM_poplog();
+    
+    return TRUE;
+   /* 
+error:
+    
+    // probably dangerous: rollback
+    env->global()->rollback();
+    
+    // clear context
+    type_engine_unload_context( env );
+    
+    // pop indent level
+    EM_poplog();
+    
+    return FALSE;
+	*/
+}
+
