@@ -152,6 +152,8 @@ Chuck_VM::~Chuck_VM()
 }
 
 
+
+
 // dac tick
 //UGEN_TICK __dac_tick( Chuck_Object * SELF, SAMPLE in, SAMPLE * out ) 
 //{ *out = in; return TRUE; }
@@ -243,7 +245,8 @@ t_CKBOOL Chuck_VM::set_priority( t_CKINT priority, Chuck_VM * vm )
 t_CKBOOL Chuck_VM::initialize( t_CKBOOL enable_audio, t_CKBOOL halt, t_CKUINT srate,
                                t_CKUINT buffer_size, t_CKUINT num_buffers,
                                t_CKUINT dac, t_CKUINT adc, t_CKUINT dac_chan,
-                               t_CKUINT adc_chan, t_CKBOOL block )
+                               t_CKUINT adc_chan, t_CKBOOL block, t_CKUINT adaptive,
+                               t_CKBOOL force_srate )
 {
     if( m_init )
     {
@@ -258,23 +261,56 @@ t_CKBOOL Chuck_VM::initialize( t_CKBOOL enable_audio, t_CKBOOL halt, t_CKUINT sr
     // log
     EM_log( CK_LOG_SYSTEM, "initializing virtual machine..." );
     EM_pushlog(); // push stack
-    EM_log( CK_LOG_SYSTEM, "behavior: %s", halt ? "HALT" : "LOOP" );
-
-    // lockdown
-    Chuck_VM_Object::lock_all();
 
     // allocate bbq
     m_bbq = new BBQ;
     m_halt = halt;
     m_audio = enable_audio;
     m_block = block;
+    m_num_adc_channels = adc_chan;
+    m_num_dac_channels = dac_chan;
+
+    // set some parameters
+    m_bbq->set_srate( srate );
+    m_bbq->set_bufsize( buffer_size );
+    m_bbq->set_numbufs( num_buffers );
+    m_bbq->set_inouts( adc, dac );
+    m_bbq->set_chans( adc_chan, dac_chan );
+
+    // log
+    EM_log( CK_LOG_SYSTEM, "probing '%s' audio subsystem...", m_audio ? "real-time" : "fake-time" );
+
+    // probe / init (this shouldn't start audio yet - moved here 1.3.1.2)
+    if( !m_bbq->initialize( m_num_dac_channels, m_num_adc_channels,
+        Digitalio::m_sampling_rate, 16, 
+        Digitalio::m_buffer_size, Digitalio::m_num_buffers,
+        Digitalio::m_dac_n, Digitalio::m_adc_n,
+        m_block, this, m_audio, NULL, NULL, force_srate ) )
+    {
+        m_last_error = "cannot initialize audio device (use --silent/-s for non-realtime)";
+        // pop indent
+        EM_poplog();
+        // clean up
+        SAFE_DELETE( m_bbq );
+        return FALSE;
+    }
     
+    // update parameters that could have been changed during probe (1.3.1.2)
+    srate = Digitalio::sampling_rate();
+    buffer_size = Digitalio::buffer_size();
+    dac = Digitalio::dac_num();
+    adc = Digitalio::adc_num();
+
+    // lockdown
+    Chuck_VM_Object::lock_all();
+
     // log
     EM_log( CK_LOG_SYSTEM, "allocating shreduler..." );
     // allocate shreduler
     m_shreduler = new Chuck_VM_Shreduler;
     m_shreduler->bbq = m_bbq;
     m_shreduler->rt_audio = enable_audio;
+    m_shreduler->set_adaptive( adaptive > 0 ? adaptive : 0 );
 
     // log
     EM_log( CK_LOG_SYSTEM, "allocating messaging buffers..." );
@@ -288,8 +324,9 @@ t_CKBOOL Chuck_VM::initialize( t_CKBOOL enable_audio, t_CKBOOL halt, t_CKUINT sr
     m_event_buffer = new CBufferSimple;
     m_event_buffer->initialize( 1024, sizeof(Chuck_Event *) );
     //m_event_buffer->join(); // this should also return 0
-
+    
     // log
+    EM_log( CK_LOG_SYSTEM, "behavior: %s", halt ? "HALT" : "LOOP" );
     EM_log( CK_LOG_SYSTEM, "real-time audio: %s", enable_audio ? "YES" : "NO" );
     EM_log( CK_LOG_SYSTEM, "mode: %s", block ? "BLOCKING" : "CALLBACK" );
     EM_log( CK_LOG_SYSTEM, "sample rate: %ld", srate );
@@ -297,18 +334,10 @@ t_CKBOOL Chuck_VM::initialize( t_CKBOOL enable_audio, t_CKBOOL halt, t_CKUINT sr
     if( enable_audio )
     {
         EM_log( CK_LOG_SYSTEM, "num buffers: %ld", num_buffers );
-        EM_log( CK_LOG_SYSTEM, "devices adc: %ld dac: %d (default 0)", adc, dac );
+        EM_log( CK_LOG_SYSTEM, "adc: %ld dac: %d", adc, dac );
+        EM_log( CK_LOG_SYSTEM, "adaptive block processing: %ld", adaptive > 1 ? adaptive : 0 ); 
     }
     EM_log( CK_LOG_SYSTEM, "channels in: %ld out: %ld", adc_chan, dac_chan );
-    m_num_adc_channels = adc_chan;
-    m_num_dac_channels = dac_chan;
-
-    // at least set the sample rate and buffer size
-    m_bbq->set_srate( srate );
-    m_bbq->set_bufsize( buffer_size );
-    m_bbq->set_numbufs( num_buffers );
-    m_bbq->set_inouts( adc, dac );
-    m_bbq->set_chans( adc_chan, dac_chan );
 
     // pop log
     EM_poplog();
@@ -357,9 +386,10 @@ t_CKBOOL Chuck_VM::initialize_synthesis( )
     g_t_dac->ugen_info->num_outs = 
         g_t_dac->ugen_info->num_ins = m_num_dac_channels;
     m_dac = (Chuck_UGen *)instantiate_and_initialize_object( g_t_dac, NULL );
-    object_ctor( m_dac, NULL, NULL ); // TODO: this can't be the place to do this
-    stereo_ctor( m_dac, NULL, NULL ); // TODO: is the NULL shred a problem?
-    multi_ctor( m_dac, NULL, NULL );  // TODO: remove and let type system do this
+    // Chuck_DL_Api::Api::instance() added 1.3.0.0
+    object_ctor( m_dac, NULL, NULL, Chuck_DL_Api::Api::instance() ); // TODO: this can't be the place to do this
+    stereo_ctor( m_dac, NULL, NULL, Chuck_DL_Api::Api::instance() ); // TODO: is the NULL shred a problem?
+    multi_ctor( m_dac, NULL, NULL, Chuck_DL_Api::Api::instance() );  // TODO: remove and let type system do this
     m_dac->add_ref();
     // lock it
     m_dac->lock();
@@ -369,9 +399,10 @@ t_CKBOOL Chuck_VM::initialize_synthesis( )
     g_t_adc->ugen_info->num_ins = 
         g_t_adc->ugen_info->num_outs = m_num_adc_channels;
     m_adc = (Chuck_UGen *)instantiate_and_initialize_object( g_t_adc, NULL );
-    object_ctor( m_adc, NULL, NULL ); // TODO: this can't be the place to do this
-    stereo_ctor( m_adc, NULL, NULL );
-    multi_ctor( m_adc, NULL, NULL ); // TODO: remove and let type system do this
+    // Chuck_DL_Api::Api::instance() added 1.3.0.0
+    object_ctor( m_adc, NULL, NULL, Chuck_DL_Api::Api::instance() ); // TODO: this can't be the place to do this
+    stereo_ctor( m_adc, NULL, NULL, Chuck_DL_Api::Api::instance() );
+    multi_ctor( m_adc, NULL, NULL, Chuck_DL_Api::Api::instance() ); // TODO: remove and let type system do this
     m_adc->add_ref();
     // lock it
     m_adc->lock();
@@ -383,27 +414,13 @@ t_CKBOOL Chuck_VM::initialize_synthesis( )
     m_bunghole->lock();
     initialize_object( m_bunghole, &t_ugen );
     m_bunghole->tick = NULL;
+    m_bunghole->alloc_v( m_shreduler->m_max_block_size );
     m_shreduler->m_dac = m_dac;
     m_shreduler->m_adc = m_adc;
     m_shreduler->m_bunghole = m_bunghole;
     m_shreduler->m_num_dac_channels = m_num_dac_channels;
     m_shreduler->m_num_adc_channels = m_num_adc_channels;
 
-    // log
-    EM_log( CK_LOG_SYSTEM, "initializing '%s' audio...", m_audio ? "real-time" : "fake-time" );
-    // init bbq
-    if( !m_bbq->initialize( m_num_dac_channels, m_num_adc_channels,
-        Digitalio::m_sampling_rate, 16, 
-        Digitalio::m_buffer_size, Digitalio::m_num_buffers,
-        Digitalio::m_dac_n, Digitalio::m_adc_n,
-        m_block, this, m_audio ) )
-    {
-        m_last_error = "cannot initialize audio device (try using --silent/-s)";
-        // pop indent
-        EM_poplog();
-        return FALSE;
-    }
-
     // pop indent
     EM_poplog();
 
@@ -668,6 +685,14 @@ t_CKBOOL Chuck_VM::compute()
         // broadcast queued events
         while( m_event_buffer->get( &event, 1 ) )
         { event->broadcast(); iterate = TRUE; }
+        
+        // loop over thread-specific queued event buffers (added 1.3.0.0)
+        for( list<CBufferSimple *>::const_iterator i = m_event_buffers.begin();
+             i != m_event_buffers.end(); i++ )
+        {
+            while( (*i)->get( &event, 1 ) )
+            { event->broadcast(); iterate = TRUE; }
+        }
 
         // process messages
         while( m_msg_buffer->get( &msg, 1 ) )
@@ -700,10 +725,12 @@ t_CKBOOL Chuck_VM::run( t_CKINT num_samps )
         if( !m_audio_started ) start_audio();
 
         // advance the shreduler
-        m_shreduler->advance();
-
-        // count
-        if( num_samps > 0 ) num_samps--;
+        if( !m_shreduler->m_adaptive )
+        {
+            m_shreduler->advance();
+            if( num_samps > 0 ) num_samps--;
+        }
+        else m_shreduler->advance_v( num_samps );
     }
 
     return FALSE;
@@ -791,23 +818,64 @@ t_CKBOOL Chuck_VM::queue_msg( Chuck_Msg * msg, int count )
 
 //-----------------------------------------------------------------------------
 // name: queue_event()
-// desc: ...
+// desc: since 1.3.0.0 a buffer is passed in associated with each thread
 //-----------------------------------------------------------------------------
-t_CKBOOL Chuck_VM::queue_event( Chuck_Event * event, int count )
+t_CKBOOL Chuck_VM::queue_event( Chuck_Event * event, int count, 
+                                CBufferSimple * buffer )
 {
+    // sanity
     assert( count == 1 );
-    m_event_buffer->put( &event, count );
+    // if null
+    if( buffer == NULL )
+    {
+        // use non thread-safe buffer
+        buffer = m_event_buffer;
+    }
+    // put into the buffer
+    buffer->put( &event, count );
+
+    // done
     return TRUE;
 }
 
 
 
 
+//-----------------------------------------------------------------------------
+// name: create_event_buffer()
+// desc: added 1.3.0.0 to fix uber-crash
+//-----------------------------------------------------------------------------
+CBufferSimple * Chuck_VM::create_event_buffer()
+{
+    CBufferSimple * buffer = new CBufferSimple;
+    buffer->initialize( 1024, sizeof(Chuck_Event *) );
+    m_event_buffers.push_back(buffer);
+    
+    return buffer;
+}
+
+
+
+
+//-----------------------------------------------------------------------------
+// name: destroy_event_buffer()
+// desc: added 1.3.0.0 to fix uber-crash
+//-----------------------------------------------------------------------------
+void Chuck_VM::destroy_event_buffer( CBufferSimple * buffer )
+{
+    m_event_buffers.remove( buffer );
+    delete buffer;
+}
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: get_reply()
 // desc: ...
+// TODO: make thread safe for multiple consumers
 //-----------------------------------------------------------------------------
-Chuck_Msg * Chuck_VM::get_reply( )
+Chuck_Msg * Chuck_VM::get_reply()
 {
     Chuck_Msg * msg = NULL;
     m_reply_buffer->get( &msg, 1 );
@@ -820,6 +888,7 @@ Chuck_Msg * Chuck_VM::get_reply( )
 //-----------------------------------------------------------------------------
 // name: process_msg()
 // desc: ...
+// TODO: make thread safe for multiple consumers
 //-----------------------------------------------------------------------------
 t_CKUINT Chuck_VM::process_msg( Chuck_Msg * msg )
 {
@@ -1005,7 +1074,7 @@ t_CKUINT Chuck_VM::process_msg( Chuck_Msg * msg )
     {
         t_CKUINT n = m_shreduler->highest();
         m_shred_id = n;
-        fprintf( stderr, "[chuck](VM): reseting shred id to %d...\n", m_shred_id + 1 );
+        fprintf( stderr, "[chuck](VM): reseting shred id to %lu...\n", m_shred_id + 1 );
     }
 
 done:
@@ -1377,7 +1446,7 @@ out_of_memory:
 
     // we have a problem
     fprintf( stderr, 
-        "[chuck](VM): OutOfMemory: while allocating stack '%s'\n" );
+        "[chuck](VM): OutOfMemory: while allocating stack\n" );
 
     // return FALSE
     return FALSE;
@@ -1417,7 +1486,7 @@ Chuck_VM_Shred::Chuck_VM_Shred()
 {
     mem = new Chuck_VM_Stack;
     reg = new Chuck_VM_Stack;
-    code = NULL;
+    code = code_orig = NULL;
     next = prev = NULL;
     instr = NULL;
     parent = NULL;
@@ -1494,15 +1563,57 @@ t_CKBOOL Chuck_VM_Shred::initialize( Chuck_VM_Code * c,
 //-----------------------------------------------------------------------------
 t_CKBOOL Chuck_VM_Shred::shutdown()
 {
+    // spencer - March 2012 (added 1.3.0.0)
+    // can't dealloc ugens while they are still keys to a map; 
+    // add reference, store them in a vector, and release them after
+    // SPENCERTODO: is there a better way to do this????
+    std::vector<Chuck_UGen *> release_v;
+    release_v.reserve(m_ugen_map.size());
+    
     // get iterator to our map
     map<Chuck_UGen *, Chuck_UGen *>::iterator iter = m_ugen_map.begin();
     while( iter != m_ugen_map.end() )
     {
-        (*iter).first->disconnect( TRUE );
+        // get the ugen
+        Chuck_UGen * ugen = iter->first;
+        // CK_GC_LOG("Chuck_VM_Shred::shutdown() disconnect: 0x%08x", ugen);
+        
+        // store ref in array for now (added 1.3.0.0)
+        release_v.push_back(ugen);
+        // no need to bump reference since now ugen_map ref counts
+        // ugen->add_ref();
+        
+        // disconnect
+        ugen->disconnect( TRUE );
+
+        // advance the iterator
         iter++;
     }
+
+    // clear map
     m_ugen_map.clear();
 
+    // loop over vector
+    for( vector<Chuck_UGen *>::iterator rvi = release_v.begin(); 
+         rvi != release_v.end(); rvi++ )
+    {
+        // release it
+        (*rvi)->release();
+    }
+    
+    // loop over parent object references (added 1.3.1.2)
+    for( vector<Chuck_Object *>::iterator it = m_parent_objects.begin();
+         it != m_parent_objects.end(); it++ )
+    {
+        // release it
+        (*it)->release();
+    }
+    
+    // clear the vectors (added 1.3.1.2)
+    release_v.clear();
+    m_parent_objects.clear();
+
+    // reclaim the stacks
     SAFE_DELETE( mem );
     SAFE_DELETE( reg );
     base_ref = NULL;
@@ -1512,10 +1623,13 @@ t_CKBOOL Chuck_VM_Shred::shutdown()
     // obj_array_size = 0;
 
     // TODO: is this right?
-    code_orig->release();
+    if( code_orig )
+        code_orig->release();
+    // clear it
     code_orig = code = NULL;
-    // what to do with next and prev?
 
+    // TODO: what to do with next and prev?
+    
     return TRUE;
 }
 
@@ -1528,9 +1642,15 @@ t_CKBOOL Chuck_VM_Shred::shutdown()
 //-----------------------------------------------------------------------------
 t_CKBOOL Chuck_VM_Shred::add( Chuck_UGen * ugen )
 {
-    if( m_ugen_map[ugen] )
+    if( !ugen || m_ugen_map[ugen] )
         return FALSE;
 
+    // increment reference count (added 1.3.0.0)
+    ugen->add_ref();
+    
+    // RUBBISH
+    // cerr << "vm add ugen: 0x" << hex << (int)ugen << endl;
+
     m_ugen_map[ugen] = ugen;
     return TRUE;
 }
@@ -1544,9 +1664,15 @@ t_CKBOOL Chuck_VM_Shred::add( Chuck_UGen * ugen )
 //-----------------------------------------------------------------------------
 t_CKBOOL Chuck_VM_Shred::remove( Chuck_UGen * ugen )
 {
-    if( !m_ugen_map[ugen] )
+    if( !ugen || !m_ugen_map[ugen] )
         return FALSE;
+
+    // decrement reference count (added 1.3.0.0)
+    ugen->release();
     
+    // RUBBISH
+    // cerr << "vm remove ugen: 0x" << hex << (int)ugen << endl;
+
     // remove it
     m_ugen_map.erase( ugen );
     return TRUE;
@@ -1555,6 +1681,26 @@ t_CKBOOL Chuck_VM_Shred::remove( Chuck_UGen * ugen )
 
 
 
+//-----------------------------------------------------------------------------
+// name: add_parent_ref()
+// desc: add parent object reference (added 1.3.1.2)
+//-----------------------------------------------------------------------------
+t_CKVOID Chuck_VM_Shred::add_parent_ref( Chuck_Object * obj )
+{
+    // sanity check
+    if( !obj )
+        return;
+    
+    // reference count
+    obj->add_ref();
+    
+    // add it to vector
+    m_parent_objects.push_back( obj );
+}
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: run()
 // desc: ...
@@ -1595,7 +1741,6 @@ t_CKBOOL Chuck_VM_Shred::run( Chuck_VM * vm )
 
 
 
-
 //-----------------------------------------------------------------------------
 // name: Chuck_VM_Shreduler()
 // desc: ...
@@ -1612,6 +1757,8 @@ Chuck_VM_Shreduler::Chuck_VM_Shreduler()
     m_bunghole = NULL;
     m_num_dac_channels = 0;
     m_num_adc_channels = 0;
+    
+    set_adaptive( 0 );
 }
 
 
@@ -1653,6 +1800,20 @@ t_CKBOOL Chuck_VM_Shreduler::shutdown()
 
 
 
+//-----------------------------------------------------------------------------
+// name: set_adaptive()
+// desc: ...
+//-----------------------------------------------------------------------------
+void Chuck_VM_Shreduler::set_adaptive( t_CKUINT max_block_size )
+{
+    m_max_block_size = max_block_size > 1 ? max_block_size : 0;
+    m_adaptive = m_max_block_size > 1;
+    m_samps_until_next = -1;
+}
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: add_blocked()
 // desc: add shred to the shreduler's blocked list
@@ -1765,11 +1926,98 @@ t_CKBOOL Chuck_VM_Shreduler::shredule( Chuck_VM_Shred * shred,
         }
     }
 
+    t_CKTIME diff = shred_list->wake_time - this->now_system;
+    if( diff < 0 ) diff = 0;
+    // if( diff < m_samps_until_next )
+    m_samps_until_next = diff;
+    
     return TRUE;
 }
 
 
 
+
+//-----------------------------------------------------------------------------
+// name: advance_v()
+// desc: ...
+//-----------------------------------------------------------------------------
+void Chuck_VM_Shreduler::advance_v( t_CKINT & numLeft )
+{
+    t_CKINT i, j, numFrames;
+    SAMPLE frame[128], gain[128], sum;
+    BBQ * audio = this->bbq;
+    
+    // compute number of frames to compute; update
+    numFrames = ck_min( m_max_block_size, numLeft );
+    if( this->m_samps_until_next >= 0 )
+    {
+        numFrames = (t_CKINT)(ck_min( numFrames, this->m_samps_until_next ));
+        if( numFrames == 0 ) numFrames = 1;
+        this->m_samps_until_next -= numFrames;
+    }
+    numLeft -= numFrames;
+
+    // advance system 'now'
+    this->now_system += numFrames;
+
+    // tick in
+    if( rt_audio )
+    {
+        for( j = 0; j < m_num_adc_channels; j++ )
+        {
+            // update channel
+            m_adc->m_multi_chan[j]->m_time = this->now_system;
+            // cache gain
+            gain[j] = m_adc->m_multi_chan[j]->m_gain;
+        }
+        
+        // adaptive block
+        for( i = 0; i < numFrames; i++ )
+        {
+            // get input
+            audio->digi_in()->tick_in( frame, m_num_adc_channels );
+            // clear
+            sum = 0.0f;
+            // loop over channels
+            for( j = 0; j < m_num_adc_channels; j++ )
+            {
+                m_adc->m_multi_chan[j]->m_current_v[i] = frame[j] * gain[j] * m_adc->m_gain;
+                sum += m_adc->m_multi_chan[j]->m_current_v[i];
+            }
+            m_adc->m_current_v[i] = sum / m_num_adc_channels;
+        }
+        
+        for( j = 0; j < m_num_adc_channels; j++ )
+        {
+            // update last
+            m_adc->m_multi_chan[j]->m_last = m_adc->m_multi_chan[j]->m_current_v[numFrames-1];
+        }
+        // update last
+        m_adc->m_last = m_adc->m_current_v[numFrames-1];
+        // update time
+        m_adc->m_time = this->now_system;
+    }
+
+    // dac
+    m_dac->system_tick_v( this->now_system, numFrames );
+
+    // suck samples
+    m_bunghole->system_tick_v( this->now_system, numFrames );
+
+    // adaptive block
+    for( i = 0; i < numFrames; i++ )
+    {
+        for( j = 0; j < m_num_dac_channels; j++ )
+            frame[j] = m_dac->m_multi_chan[j]->m_current_v[i];
+        
+        // tick
+        audio->digi_out()->tick_out( frame, m_num_dac_channels );
+    }
+}
+
+
+
+
 //-----------------------------------------------------------------------------
 // name: advance2()
 // desc: ...
@@ -1837,6 +2085,7 @@ void Chuck_VM_Shreduler::advance( )
         for( i = 0; i < m_num_adc_channels; i++ )
         {
             m_adc->m_multi_chan[i]->m_current = frame[i] * m_adc->m_multi_chan[i]->m_gain * m_adc->m_gain;
+            m_adc->m_multi_chan[i]->m_last = m_adc->m_multi_chan[i]->m_current;
             m_adc->m_multi_chan[i]->m_time = this->now_system;
             sum += m_adc->m_multi_chan[i]->m_current;
         }
@@ -1847,7 +2096,7 @@ void Chuck_VM_Shreduler::advance( )
     // dac
     m_dac->system_tick( this->now_system );
     for( i = 0; i < m_num_dac_channels; i++ )
-        frame[i] = m_dac->m_multi_chan[i]->m_current * .5f;
+        frame[i] = m_dac->m_multi_chan[i]->m_current; // * .5f;
 
     // suck samples
     m_bunghole->system_tick( this->now_system );
@@ -1858,6 +2107,7 @@ void Chuck_VM_Shreduler::advance( )
 
 
 
+
 //-----------------------------------------------------------------------------
 // name: get()
 // desc: ...
@@ -1868,7 +2118,10 @@ Chuck_VM_Shred * Chuck_VM_Shreduler::get( )
 
     // list empty
     if( !shred )
+    {
+        m_samps_until_next = -1;
         return NULL;
+    }
 
     // TODO: should this be <=?
     if( shred->wake_time <= ( this->now_system + .5 ) )
@@ -1881,7 +2134,11 @@ Chuck_VM_Shred * Chuck_VM_Shreduler::get( )
         shred->prev = NULL;
         
         if( shred_list )
+        {
             shred_list->prev = NULL;
+            m_samps_until_next = shred_list->wake_time - this->now_system;
+            if( m_samps_until_next < 0 ) m_samps_until_next = 0;
+        }
 
         return shred;
     }
@@ -2079,7 +2336,7 @@ void Chuck_VM_Shreduler::status( Chuck_VM_Status * status )
     }
 
     // get current shred
-    if( temp = m_current_shred )
+    if( ( temp = m_current_shred ) )
         list.push_back( temp );
 
     // sort the list
