@@ -131,7 +131,7 @@ CK_DLL_MFUN( LPC_cget_power );
 CK_DLL_MFUN( LPC_ctrl_coefs );
 CK_DLL_MFUN( LPC_cget_coefs );
 // offset
-static t_CKUINT LPC_offset_data = 0;
+//static t_CKUINT LPC_offset_data = 0;
 
 
 // utility functions
@@ -1064,7 +1064,7 @@ struct Corr_Object
         if( fbuf == NULL || gbuf == NULL || buffy == NULL )
         {
             // error
-            fprintf( stderr, "[chuck]: Corr failed to allocate %d-element buffer(s)...",
+            fprintf( stderr, "[chuck]: Corr failed to allocate %ld-element buffer(s)...",
                 mincap );
             // clean up
             this->reset();
