@@ -71,7 +71,7 @@ CBufferAdvance::~CBufferAdvance()
 // name: initialize()
 // desc: initialize
 //-----------------------------------------------------------------------------
-BOOL__ CBufferAdvance::initialize( UINT__ num_elem, UINT__ width )
+BOOL__ CBufferAdvance::initialize( UINT__ num_elem, UINT__ width, CBufferSimple * event_buffer )
 {
     // cleanup
     cleanup();
@@ -85,6 +85,8 @@ BOOL__ CBufferAdvance::initialize( UINT__ num_elem, UINT__ width )
     //m_read_offset = 0;
     m_write_offset = 0;
     m_max_elem = (SINT__)num_elem;
+    
+    m_event_buffer = event_buffer;
 
     return true;
 }
@@ -154,7 +156,7 @@ UINT__ CBufferAdvance::join( Chuck_Event * event )
 void CBufferAdvance::resign( UINT__ read_offset_index )
 {
     // make sure read_offset_index passed in is valid
-    if( read_offset_index < 0 || read_offset_index >= m_read_offsets.size() )
+    if( read_offset_index >= m_read_offsets.size() )
         return;
 
     // TODO: necessary?
@@ -165,6 +167,7 @@ void CBufferAdvance::resign( UINT__ read_offset_index )
 
     // "invalidate" the pointer at that index
     m_read_offsets[read_offset_index].read_offset = -1;
+    m_read_offsets[read_offset_index].event = NULL;
 
     // TODO: necessary?
     m_mutex.release();
@@ -231,7 +234,7 @@ void CBufferAdvance::put( void * data, UINT__ num_elem )
             }
 
             if( m_read_offsets[j].event )
-                m_read_offsets[j].event->queue_broadcast();
+                m_read_offsets[j].event->queue_broadcast( m_event_buffer );
         }
     }
 
